package vc.thinker.absctacts.protocolproxy.dto;

import com.google.common.collect.Maps;
import lombok.Getter;
import vc.thinker.absctacts.protocolproxy.enums.IMeterDataDefine;

import java.time.LocalDateTime;
import java.util.Map;

/**
 * @author HeTongHao
 * @since 2021/7/9 17:47
 */
@Getter
public class MeterReportEntity implements IMeterReportEntity {
    private MeterReportEntity(String deviceCode, LocalDateTime collectTime, Map<IMeterDataDefine, Object> dataMap) {
        this.deviceCode = deviceCode;
        this.collectTime = collectTime;
        this.dataMap = dataMap;
    }

    private final String deviceCode;
    private final LocalDateTime collectTime;
    private final Map<IMeterDataDefine, Object> dataMap;

    public static MeterReportEntityBuilder builder(String deviceId, LocalDateTime collectTime) {
        return new MeterReportEntityBuilder(deviceId, collectTime);
    }

    public static class MeterReportEntityBuilder {
        private MeterReportEntityBuilder(String deviceId, LocalDateTime collectTime) {
            this.deviceId = deviceId;
            this.collectTime = collectTime;
        }

        private final String deviceId;
        private final LocalDateTime collectTime;
        private final Map<IMeterDataDefine, Object> dataMap = Maps.newConcurrentMap();

        public <DD extends IMeterDataDefine> MeterReportEntityBuilder putValue(DD dataDefine, Object value) {
            putData(dataDefine, value);
            return this;
        }

        public <DD extends IMeterDataDefine> MeterReportEntityBuilder putValues(Map<DD, Object> dataDefineAndValueMap) {
            dataDefineAndValueMap.forEach(this::putData);
            return this;
        }

        private <DD extends IMeterDataDefine> void putData(DD dataDefine, Object value) {
            dataMap.put(dataDefine, value);
        }

        public MeterReportEntity build() {
            return new MeterReportEntity(deviceId, collectTime, dataMap);
        }
    }
}
